==> alsa-driver-0.5.5.tar.bz2: alsa-driver-0.5.5/lowlevel/sb/sb8_init.c <==
/*
 *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
 *  Routines for control of SoundBlaster cards
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * --
 *
 * Wed Aug 18 15:39:50 BST 1999 George David Morrison <gdm@gedamo.demon.co.uk>
 *   In snd_sb8dsp_new_device, both playback_pchn1 and capture_pchn1 were set
 *   to point to pcm->playback.private_data causing playback to fail with DMA
 *   errors. Fixed.
 */

#define SND_MAIN_OBJECT_FILE
#include "../../include/driver.h"
#include "../../include/sb.h"

int snd_sb8dsp_command(sbdsp_t * codec, unsigned char val)
{
	int i;
	unsigned long limit;

	limit = jiffies + HZ / 10;
	for (i = 500000; i > 0 && (limit-jiffies) > 0; i--)
		if ((inb(SBP(codec, STATUS)) & 0x80) == 0) {
			outb(val, SBP(codec, COMMAND));
			return 1;
		}
	snd_printd(__FUNCTION__ ": timeout (0x%x)\n", val);
	return 0;
}

int snd_sb8dsp_get_byte(sbdsp_t * codec)
{
	int i;

	for (i = 1000; i; i--)
		if (inb(SBP(codec, DATA_AVAIL)) & 0x80)
			return inb(SBP(codec, READ));
	snd_printd(__FUNCTION__ " failed: 0x%x = 0x%x!!!\n", SBP(codec, DATA_AVAIL), inb(SBP(codec, DATA_AVAIL)));
	return -ENODEV;
}

int snd_sb8dsp_reset(sbdsp_t * codec)
{
	int i;

	outb(1, SBP(codec, RESET));
	udelay(10);
	outb(0, SBP(codec, RESET));
	udelay(30);
	for (i = 0; i < 1000 && !(inb(SBP(codec, DATA_AVAIL)) & 0x80); i++);
	if (inb(SBP(codec, READ)) != 0xaa) {
		snd_printd(__FUNCTION__ ": failed at 0x%x!!!\n", codec->port);
		return -ENODEV;
	}
	return 0;
}

int snd_sb8dsp_version(snd_pcm_t * pcm)
{
	unsigned long flags;
	sbdsp_t *codec;
	int i;
	unsigned int result = -1;

	codec = snd_magic_cast(sbdsp_t, pcm->private_data, -ENXIO);
	spin_lock_irqsave(&codec->reg_lock, flags);
	snd_sb8dsp_command(codec, SB_DSP_GET_VERSION);
	for (i = 100000; i; i--) {
		if (inb(SBP(codec, DATA_AVAIL)) & 0x80) {
			result = (short) inb(SBP(codec, READ)) << 8;
			break;
		}
	}
	for (i = 100000; i; i--) {
		if (inb(SBP(codec, DATA_AVAIL)) & 0x80) {
			result |= (short) inb(SBP(codec, READ));
			break;
		}
	}
	spin_unlock_irqrestore(&codec->reg_lock, flags);
	return result;
}

int snd_sb8dsp_probe(snd_pcm_t * pcm)
{
	unsigned long flags;
	sbdsp_t *codec;
	int version, hw;
	char *str;

	/*
	 *  initialization sequence
	 */

	codec = snd_magic_cast(sbdsp_t, pcm->private_data, -ENXIO);

	spin_lock_irqsave(&codec->reg_lock, flags);
	if (snd_sb8dsp_reset(codec) < 0) {
		snd_printdd("SB: [0x%x] reset failed... 0x%x\n", codec->port, inb(SBP(codec, READ)));
		spin_unlock_irqrestore(&codec->reg_lock, flags);
		return -ENODEV;
	}
	spin_unlock_irqrestore(&codec->reg_lock, flags);

	version = snd_sb8dsp_version(pcm);
	if (version < 0)
		return -ENODEV;

	snd_printdd("SB [0x%x]: DSP chip found, version = %i.%i\n", codec->port, version >> 8, version & 0xff);

	hw = SB_HW_AUTO;
	switch (version >> 8) {
	case 1:
		hw = SB_HW_10;
		break;
	case 2:
		hw = !(version & 0xff) ? SB_HW_20 : SB_HW_201;
		break;
	case 3:
		hw = SB_HW_PRO;
		break;
	case 4:
		snd_printk("SB [0x%x]: DSP chip version %i.%i is not supported by the SB8 code\n", codec->port, version >> 8, version & 0xff);
		break;
	default:
		snd_printk("SB [0x%x]: unknown DSP chip version %i.%i\n", codec->port, version >> 8, version & 0xff);
		return -ENODEV;
	}
	switch (codec->hardware = hw) {
	case SB_HW_10:
		str = "Sound Blaster 1.0";
		break;
	case SB_HW_20:
		str = "Sound Blaster 2.0";
		break;
	case SB_HW_201:
		str = "Sound Blaster 2.01+";
		break;
	case SB_HW_PRO:
		str = "Sound Blaster Pro";
		break;
	default:
		str = "???";
	}
	strcpy(codec->name, str);
	sprintf(pcm->name, "DSP v%i.%i", version >> 8, version & 0xff);

	return 0;
}

void snd_sb8dsp_free(void *private_data)
{
	sbdsp_t *codec = snd_magic_cast(sbdsp_t, private_data, );
	snd_magic_kfree(codec);
}

int snd_sb8dsp_new_pcm(snd_card_t * card,
		       int device,
		       unsigned short port,
		       snd_irq_t * irqptr,
		       snd_dma_t * dma8ptr,
		       unsigned short hardware,
		       snd_pcm_t ** rpcm)
{
	snd_pcm_t *pcm;
	sbdsp_t *codec;
	int err;

	if ((err = snd_pcm_new(card, "SB8 DSP", device, 1, 1, &pcm)) < 0)
		return err;
	codec = snd_magic_kcalloc(sbdsp_t, 0, GFP_KERNEL);
	if (codec == NULL)
		return -ENOMEM;
	codec->reg_lock = SPIN_LOCK_UNLOCKED;
	codec->open8_lock = SPIN_LOCK_UNLOCKED;
	codec->open16_lock = SPIN_LOCK_UNLOCKED;
	codec->midi_input_lock = SPIN_LOCK_UNLOCKED;
	codec->mixer.lock = SPIN_LOCK_UNLOCKED;
	codec->pcm = pcm;
	codec->card = pcm->card;
	codec->port = port;
	codec->mixer.port = port;
	codec->irqptr = irqptr;
	codec->irq = irqptr->irq;
	codec->dma8ptr = dma8ptr;
	if (dma8ptr)
		codec->dma8 = dma8ptr->dma;
	codec->hardware = hardware;
	pcm->private_data = codec;
	pcm->private_free = snd_sb8dsp_free;
	strcpy(pcm->name, "Sound Blaster");
	if (snd_sb8dsp_probe(pcm) < 0) {
		snd_device_free(card, pcm);
		return -ENODEV;
	}
	pcm->info_flags = SND_PCM_INFO_PLAYBACK | SND_PCM_INFO_CAPTURE;

	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].private_data = codec;
	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].open = snd_sb8_playback_open;
	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].close = snd_sb8_playback_close;
	pcm->chn[SND_PCM_CHANNEL_CAPTURE].private_data = codec;
	pcm->chn[SND_PCM_CHANNEL_CAPTURE].open = snd_sb8_capture_open;
	pcm->chn[SND_PCM_CHANNEL_CAPTURE].close = snd_sb8_capture_close;

	*rpcm = pcm;
	return 0;
}

EXPORT_SYMBOL(snd_sb8dsp_new_pcm);
  /* sb8.c */
EXPORT_SYMBOL(snd_sb8dsp_interrupt);
  /* mixer8.c */
EXPORT_SYMBOL(snd_sb8mixer_read);
EXPORT_SYMBOL(snd_sb8mixer_write);
EXPORT_SYMBOL(snd_sb8dsp_new_mixer);
  /* midi8.c */
EXPORT_SYMBOL(snd_sb8dsp_midi_interrupt);
EXPORT_SYMBOL(snd_sb8dsp_midi_new);

/*
 *  INIT part
 */

#ifdef MODULE

int __init init_module(void)
{
	return 0;
}

void __exit cleanup_module(void)
{
}

#endif

==> alsa-driver-0.5.8b.tar.bz2: alsa-driver-0.5.8b/lowlevel/sb/sb8_init.c <==
/*
 *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
 *  Routines for control of SoundBlaster cards
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * --
 *
 * Wed Aug 18 15:39:50 BST 1999 George David Morrison <gdm@gedamo.demon.co.uk>
 *   In snd_sb8dsp_new_device, both playback_pchn1 and capture_pchn1 were set
 *   to point to pcm->playback.private_data causing playback to fail with DMA
 *   errors. Fixed.
 */

#define SND_MAIN_OBJECT_FILE
#include "../../include/driver.h"
#include "../../include/sb.h"

int snd_sb8dsp_command(sbdsp_t * codec, unsigned char val)
{
	int i;
	unsigned long limit;

	limit = jiffies + HZ / 10;
	for (i = 500000; i > 0 && (limit-jiffies) > 0; i--)
		if ((inb(SBP(codec, STATUS)) & 0x80) == 0) {
			outb(val, SBP(codec, COMMAND));
			return 1;
		}
	snd_printd(__FUNCTION__ ": timeout (0x%x)\n", val);
	return 0;
}

int snd_sb8dsp_get_byte(sbdsp_t * codec)
{
	int i;

	for (i = 1000; i; i--)
		if (inb(SBP(codec, DATA_AVAIL)) & 0x80)
			return inb(SBP(codec, READ));
	snd_printd(__FUNCTION__ " failed: 0x%x = 0x%x!!!\n", SBP(codec, DATA_AVAIL), inb(SBP(codec, DATA_AVAIL)));
	return -ENODEV;
}

int snd_sb8dsp_reset(sbdsp_t * codec)
{
	int i;

	outb(1, SBP(codec, RESET));
	udelay(10);
	outb(0, SBP(codec, RESET));
	udelay(30);
	for (i = 0; i < 1000 && !(inb(SBP(codec, DATA_AVAIL)) & 0x80); i++);
	if (inb(SBP(codec, READ)) != 0xaa) {
		snd_printd(__FUNCTION__ ": failed at 0x%x!!!\n", codec->port);
		return -ENODEV;
	}
	return 0;
}

int snd_sb8dsp_version(snd_pcm_t * pcm)
{
	unsigned long flags;
	sbdsp_t *codec;
	int i;
	unsigned int result = -1;

	codec = snd_magic_cast(sbdsp_t, pcm->private_data, -ENXIO);
	spin_lock_irqsave(&codec->reg_lock, flags);
	snd_sb8dsp_command(codec, SB_DSP_GET_VERSION);
	for (i = 100000; i; i--) {
		if (inb(SBP(codec, DATA_AVAIL)) & 0x80) {
			result = (short) inb(SBP(codec, READ)) << 8;
			break;
		}
	}
	for (i = 100000; i; i--) {
		if (inb(SBP(codec, DATA_AVAIL)) & 0x80) {
			result |= (short) inb(SBP(codec, READ));
			break;
		}
	}
	spin_unlock_irqrestore(&codec->reg_lock, flags);
	return result;
}

int snd_sb8dsp_probe(snd_pcm_t * pcm)
{
	unsigned long flags;
	sbdsp_t *codec;
	int version, hw;
	char *str;

	/*
	 *  initialization sequence
	 */

	codec = snd_magic_cast(sbdsp_t, pcm->private_data, -ENXIO);

	spin_lock_irqsave(&codec->reg_lock, flags);
	if (snd_sb8dsp_reset(codec) < 0) {
		snd_printdd("SB: [0x%x] reset failed... 0x%x\n", codec->port, inb(SBP(codec, READ)));
		spin_unlock_irqrestore(&codec->reg_lock, flags);
		return -ENODEV;
	}
	spin_unlock_irqrestore(&codec->reg_lock, flags);

	version = snd_sb8dsp_version(pcm);
	if (version < 0)
		return -ENODEV;

	snd_printdd("SB [0x%x]: DSP chip found, version = %i.%i\n", codec->port, version >> 8, version & 0xff);

	hw = SB_HW_AUTO;
	switch (version >> 8) {
	case 1:
		hw = SB_HW_10;
		break;
	case 2:
		hw = !(version & 0xff) ? SB_HW_20 : SB_HW_201;
		break;
	case 3:
		hw = SB_HW_PRO;
		break;
	case 4:
		snd_printk("SB [0x%x]: DSP chip version %i.%i is not supported by the SB8 code\n", codec->port, version >> 8, version & 0xff);
		break;
	default:
		snd_printk("SB [0x%x]: unknown DSP chip version %i.%i\n", codec->port, version >> 8, version & 0xff);
		return -ENODEV;
	}
	switch (codec->hardware = hw) {
	case SB_HW_10:
		str = "Sound Blaster 1.0";
		break;
	case SB_HW_20:
		str = "Sound Blaster 2.0";
		break;
	case SB_HW_201:
		str = "Sound Blaster 2.01+";
		break;
	case SB_HW_PRO:
		str = "Sound Blaster Pro";
		break;
	default:
		str = "???";
	}
	strcpy(codec->name, str);
	sprintf(pcm->name, "DSP v%i.%i", version >> 8, version & 0xff);

	return 0;
}

void snd_sb8dsp_free(void *private_data)
{
	sbdsp_t *codec = snd_magic_cast(sbdsp_t, private_data, );
	snd_magic_kfree(codec);
}

int snd_sb8dsp_new_pcm(snd_card_t * card,
		       int device,
		       unsigned short port,
		       snd_irq_t * irqptr,
		       snd_dma_t * dma8ptr,
		       unsigned short hardware,
		       snd_pcm_t ** rpcm)
{
	snd_pcm_t *pcm;
	sbdsp_t *codec;
	int err;

	if ((err = snd_pcm_new(card, "SB8 DSP", device, 1, 1, &pcm)) < 0)
		return err;
	codec = snd_magic_kcalloc(sbdsp_t, 0, GFP_KERNEL);
	if (codec == NULL)
		return -ENOMEM;
	spin_lock_init(&codec->reg_lock);
	spin_lock_init(&codec->open8_lock);
	spin_lock_init(&codec->open16_lock);
	spin_lock_init(&codec->midi_input_lock);
	spin_lock_init(&codec->mixer.lock);
	codec->pcm = pcm;
	codec->card = pcm->card;
	codec->port = port;
	codec->mixer.port = port;
	codec->irqptr = irqptr;
	codec->irq = irqptr->irq;
	codec->dma8ptr = dma8ptr;
	if (dma8ptr)
		codec->dma8 = dma8ptr->dma;
	codec->hardware = hardware;
	pcm->private_data = codec;
	pcm->private_free = snd_sb8dsp_free;
	strcpy(pcm->name, "Sound Blaster");
	if (snd_sb8dsp_probe(pcm) < 0) {
		snd_device_free(card, pcm);
		return -ENODEV;
	}
	pcm->info_flags = SND_PCM_INFO_PLAYBACK | SND_PCM_INFO_CAPTURE;

	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].private_data = codec;
	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].open = snd_sb8_playback_open;
	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].close = snd_sb8_playback_close;
	pcm->chn[SND_PCM_CHANNEL_CAPTURE].private_data = codec;
	pcm->chn[SND_PCM_CHANNEL_CAPTURE].open = snd_sb8_capture_open;
	pcm->chn[SND_PCM_CHANNEL_CAPTURE].close = snd_sb8_capture_close;

	*rpcm = pcm;
	return 0;
}

EXPORT_SYMBOL(snd_sb8dsp_new_pcm);
  /* sb8.c */
EXPORT_SYMBOL(snd_sb8dsp_interrupt);
  /* mixer8.c */
EXPORT_SYMBOL(snd_sb8mixer_read);
EXPORT_SYMBOL(snd_sb8mixer_write);
EXPORT_SYMBOL(snd_sb8dsp_new_mixer);
  /* midi8.c */
EXPORT_SYMBOL(snd_sb8dsp_midi_interrupt);
EXPORT_SYMBOL(snd_sb8dsp_midi_new);

/*
 *  INIT part
 */

static int __init alsa_sb8_init(void)
{
	return 0;
}

static void __exit alsa_sb8_exit(void)
{
}

module_init(alsa_sb8_init)
module_exit(alsa_sb8_exit)
